package hr.unizg.srce.d470.obrasci.strategija;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class Kompozit implements Komponenta {

	private String nazivGrupe;

	private List<Komponenta> komponente;

	public Kompozit(String nazivGrupe) {
		this.nazivGrupe = nazivGrupe;
		komponente = new ArrayList<Komponenta>();
	}

	public Kompozit(Kompozit kompozit) {
		this(kompozit.nazivGrupe);
		for (Komponenta komponenta : kompozit.getKomponente())
			getKomponente().add((Komponenta) komponenta.kloniraj());
	}

	private String getNazivGrupe() {
		return nazivGrupe;
	}

	private List<Komponenta> getKomponente() {
		return komponente;
	}

	public void dodaj(Komponenta komponenta) {
		getKomponente().add(komponenta);
	}

	@Override
	public String toString() {
		return "Kompozit [nazivGrupe=" + getNazivGrupe() + "]";
	}

	@Override
	public boolean sadrzi(int broj) {
		System.out.println(this);
		for (Komponenta komponenta : getKomponente())
			if (komponenta.sadrzi(broj))
				return true;
		return false;
	}

	@Override
	public Prototip kloniraj() {
		return new Kompozit(this);
	}

	@Override
	public Iterator<Integer> iterator() {
		return new KompozitIterator(this);
	}

	private class KompozitIterator implements Iterator<Integer> {

		private Kompozit kompozit;
		private int index;
		private Iterator<Integer> podIterator;

		public KompozitIterator(Kompozit kompozit) {
			this.kompozit = kompozit;
		}

		@Override
		public boolean hasNext() {
			return index < kompozit.getKomponente().size();
		}

		@Override
		public Integer next() {
			if (podIterator == null)
				podIterator = kompozit.getKomponente().get(index).iterator();

			Integer rezultat = podIterator.next();

			if (!podIterator.hasNext()) {
				++index;
				podIterator = null;
			}

			return rezultat;
		}

	}

}
