package hr.unizg.srce.d470.vjezbe.connectx;

import java.util.ArrayList;
import java.util.Random;

public class PametnaStrategija extends Strategija {

	private final int maxDubina;

	public PametnaStrategija(int maxDubina) {
		this.maxDubina = maxDubina;
	}

	public int getMaxDubina() {
		return maxDubina;
	}

	@Override
	public Potez apply(Igra igra, Igrac igrac) {
		Random random = new Random();
		int najboljiRezultat = Integer.MIN_VALUE;
		ArrayList<Potez> najboljiPotezi = new ArrayList<>();

		int indeksProtivnika = (igra.getIgraci()[0].equals(igrac)) ? 1 : 0;

		for (Potez potez : mogucnosti(igra.getPloca(), igrac.getZeton())) {
			Igra novaIgra = igra.kopiraj();

			potez.igraj(novaIgra.getPloca());

			int trenutniRezultat = razmisli(novaIgra, indeksProtivnika, false, 1);

			System.out.println(igrac + " razmislio o potezu (" + potez + "), rezultat: " + trenutniRezultat);

			if (trenutniRezultat > najboljiRezultat) {
				najboljiRezultat = trenutniRezultat;
				najboljiPotezi = new ArrayList<>();
				najboljiPotezi.add(potez);
			} else if (trenutniRezultat == najboljiRezultat) {
				najboljiPotezi.add(potez);
			}
		}

		return najboljiPotezi.get(random.nextInt(najboljiPotezi.size()));
	}

	private int razmisli(final Igra igra, final int trenutniIgrac, final boolean maksimiziraj, final int dubina) {
		StanjeIgre stanje = igra.stanjeIgre();

		switch (stanje) {
		case POBJEDA_PRVI:
			if (trenutniIgrac == 0)
				return maksimiziraj ? 4 : 0;
			else
				return maksimiziraj ? 0 : 4;
		case POBJEDA_DRUGI:
			if (trenutniIgrac == 0)
				return maksimiziraj ? 0 : 4;
			else
				return maksimiziraj ? 4 : 0;
		case REMI:
			return 2;
		case AKTIVNA:
			break;
		default:
			throw new IllegalStateException("Nepoznato stanje igre.");
		}

		if (dubina >= getMaxDubina())
			return 1;

		int rezultat = (maksimiziraj) ? Integer.MIN_VALUE : Integer.MAX_VALUE;

		for (Potez potez : mogucnosti(igra.getPloca(), igra.getIgraci()[trenutniIgrac].getZeton())) {
			Igra novaIgra = igra.kopiraj();
			potez.igraj(novaIgra.getPloca());
			rezultat = (maksimiziraj)
					? Integer.max(rezultat,
							razmisli(novaIgra, (trenutniIgrac + 1) % 2, !maksimiziraj, dubina + 1))
					: Integer.min(rezultat,
							razmisli(novaIgra, (trenutniIgrac + 1) % 2, !maksimiziraj, dubina + 1));
		}

		return rezultat;
	}

}
