package hr.unizg.srce.d470.vjezbe.kolekcije;

import java.util.ArrayList;
import java.util.List;

public class Ploca<T> {

	private final int brRedaka;

	private final int brStupaca;

	private final List<List<T>> ploca;

	public Ploca(final int brRedaka, final int brStupaca) {
		this.brRedaka = brRedaka;
		this.brStupaca = brStupaca;

		ploca = new ArrayList<>(brStupaca);
		for (int i = 0; i < brStupaca; ++i)
			ploca.add(new ArrayList<T>(brRedaka));
	}

	public int getBrRedaka() {
		return brRedaka;
	}

	public int getBrStupaca() {
		return brStupaca;
	}

	public int getVisinaStupca(final int stupac) {
		return ploca.get(stupac).size();
	}

	public T getElement(final int redak, final int stupac) {
		try {
			return ploca.get(stupac).get(redak);
		} catch (IndexOutOfBoundsException ex) {
			return null;
		}
	}

	public void ubaciElement(final int stupac, final T element) {
		if (getVisinaStupca(stupac) >= getBrRedaka())
			throw new PopunjenStupacException("Stupac " + stupac + " je popunjen.");
		ploca.get(stupac).add(element);
	}

	public T izbaciElement(final int stupac) {
		return ploca.get(stupac).remove(0);
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for (int i = getBrRedaka() - 1; i >= 0; --i) {
			for (int j = 0; j < getBrStupaca(); ++j) {
				T element = getElement(i, j);
				sb.append((element == null) ? "." : element);
			}
			sb.append("\n");
		}
		return sb.toString();
	}

	public Ploca<T> kopiraj() {
		Ploca<T> novaPloca = new Ploca<>(getBrRedaka(), getBrStupaca());
		for (int i = 0; i < getBrStupaca(); ++i)
			for (int j = 0; j < getVisinaStupca(i); ++j)
				novaPloca.ubaciElement(i, getElement(j, i));
		return novaPloca;
	}

}
